@@ -1,4 +1,4 @@
-/* $Id: thunar-preferences-dialog.c 25741 2007-05-22 16:35:31Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2005-2007 Benedikt Meurer <benny@xfce.org>
  *
@@ -21,6 +21,8 @@
 #include <config.h>
 #endif
 
+#include <libxfce4ui/libxfce4ui.h>
+
 #include <thunar/sexy-url-label.h>
 #include <thunar/thunar-compact-view.h>
 #include <thunar/thunar-details-view.h>
@@ -33,12 +35,9 @@
 #include <thunar/thunar-preferences.h>
 #include <thunar/thunar-private.h>
 #include <thunar/thunar-util.h>
-#include <thunar/xfce-titled-dialog.h>
 
 
 
-static void thunar_preferences_dialog_class_init (ThunarPreferencesDialogClass *klass);
-static void thunar_preferences_dialog_init       (ThunarPreferencesDialog      *dialog);
 static void thunar_preferences_dialog_finalize   (GObject                      *object);
 static void thunar_preferences_dialog_response   (GtkDialog                    *dialog,
                                                   gint                          response);
@@ -59,36 +58,7 @@ struct _ThunarPreferencesDialog
 
 
 
-static GObjectClass *thunar_preferences_dialog_parent_class;
-
-
-
-GType
-thunar_preferences_dialog_get_type (void)
-{
-  static GType type = G_TYPE_INVALID;
-
-  if (G_UNLIKELY (type == G_TYPE_INVALID))
-    {
-      static const GTypeInfo info =
-      {
-        sizeof (ThunarPreferencesDialogClass),
-        NULL,
-        NULL,
-        (GClassInitFunc) thunar_preferences_dialog_class_init,
-        NULL,
-        NULL,
-        sizeof (ThunarPreferencesDialog),
-        0,
-        (GInstanceInitFunc) thunar_preferences_dialog_init,
-        NULL,
-      };
-
-      type = g_type_register_static (XFCE_TYPE_TITLED_DIALOG, I_("ThunarPreferencesDialog"), &info, 0);
-    }
-
-  return type;
-}
+G_DEFINE_TYPE (ThunarPreferencesDialog, thunar_preferences_dialog, XFCE_TYPE_TITLED_DIALOG)
 
 
 
@@ -98,7 +68,7 @@ transform_icon_size_to_index (const GValue *src_value,
                               gpointer      user_data)
 {
   GEnumClass *klass;
-  gint        n;
+  guint       n;
 
   klass = g_type_class_ref (THUNAR_TYPE_ICON_SIZE);
   for (n = 0; n < klass->n_values; ++n)
@@ -184,9 +154,6 @@ thunar_preferences_dialog_class_init (ThunarPreferencesDialogClass *klass)
   GtkDialogClass *gtkdialog_class;
   GObjectClass   *gobject_class;
 
-  /* determine the parent type class */
-  thunar_preferences_dialog_parent_class = g_type_class_peek_parent (klass);
-
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = thunar_preferences_dialog_finalize;
 
@@ -199,30 +166,27 @@ thunar_preferences_dialog_class_init (ThunarPreferencesDialogClass *klass)
 static void
 thunar_preferences_dialog_init (ThunarPreferencesDialog *dialog)
 {
-  ThunarVfsVolumeManager *volume_manager;
-  ThunarDateStyle         date_style;
-  GtkAdjustment          *adjustment;
-  GtkWidget              *notebook;
-  GtkWidget              *button;
-  GtkWidget              *align;
-  GtkWidget              *combo;
-  GtkWidget              *frame;
-  GtkWidget              *image;
-  GtkWidget              *label;
-  GtkWidget              *range;
-  GtkWidget              *table;
-  GtkWidget              *hbox;
-  GtkWidget              *ibox;
-  GtkWidget              *vbox;
-  gchar                  *path;
-  gchar                  *date;
+  ThunarDateStyle date_style;
+  GtkAdjustment  *adjustment;
+  GtkWidget      *notebook;
+  GtkWidget      *button;
+  GtkWidget      *align;
+  GtkWidget      *combo;
+  GtkWidget      *frame;
+  GtkWidget      *label;
+  GtkWidget      *range;
+  GtkWidget      *table;
+  GtkWidget      *hbox;
+  GtkWidget      *ibox;
+  GtkWidget      *vbox;
+  gchar          *path;
+  gchar          *date;
 
   /* grab a reference on the preferences */
   dialog->preferences = thunar_preferences_get ();
 
   /* configure the dialog properties */
   gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-  gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
   gtk_window_set_icon_name (GTK_WINDOW (dialog), "xfce-filemanager");
   gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
   gtk_window_set_title (GTK_WINDOW (dialog), _("File Manager Preferences"));
@@ -273,9 +237,6 @@ thunar_preferences_dialog_init (ThunarPreferencesDialog *dialog)
   gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Detailed List View"));
   gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Compact List View"));
   gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Last Active View"));
-#if !GTK_CHECK_VERSION(2,9,0)
-  g_signal_connect (G_OBJECT (combo), "changed", G_CALLBACK (g_object_notify), "active");
-#endif
   exo_mutual_binding_new_full (G_OBJECT (dialog->preferences), "default-view", G_OBJECT (combo), "active",
                                transform_view_string_to_index, transform_view_index_to_string, NULL, NULL);
   gtk_table_attach (GTK_TABLE (table), combo, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
@@ -285,14 +246,14 @@ thunar_preferences_dialog_init (ThunarPreferencesDialog *dialog)
 
   button = gtk_check_button_new_with_mnemonic (_("Sort _folders before files"));
   exo_mutual_binding_new (G_OBJECT (dialog->preferences), "misc-folders-first", G_OBJECT (button), "active");
-  thunar_gtk_widget_set_tooltip (button, _("Select this option to list folders before files when you sort a folder."));
+  gtk_widget_set_tooltip_text (button, _("Select this option to list folders before files when you sort a folder."));
   gtk_table_attach (GTK_TABLE (table), button, 0, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
   gtk_widget_show (button);
 
   button = gtk_check_button_new_with_mnemonic (_("_Show thumbnails"));
   exo_mutual_binding_new (G_OBJECT (dialog->preferences), "misc-show-thumbnails", G_OBJECT (button), "active");
-  thunar_gtk_widget_set_tooltip (button, _("Select this option to display previewable files within a "
-                                           "folder as automatically generated thumbnail icons."));
+  gtk_widget_set_tooltip_text (button, _("Select this option to display previewable files within a "
+                                         "folder as automatically generated thumbnail icons."));
   gtk_table_attach (GTK_TABLE (table), button, 0, 2, 2, 3, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
   gtk_widget_show (button);
 
@@ -314,8 +275,8 @@ thunar_preferences_dialog_init (ThunarPreferencesDialog *dialog)
 
   button = gtk_check_button_new_with_mnemonic (_("_Text beside icons"));
   exo_mutual_binding_new (G_OBJECT (dialog->preferences), "misc-text-beside-icons", G_OBJECT (button), "active");
-  thunar_gtk_widget_set_tooltip (button, _("Select this option to place the icon captions for items "
-                                           "beside the icon rather than below the icon."));
+  gtk_widget_set_tooltip_text (button, _("Select this option to place the icon captions for items "
+                                         "beside the icon rather than below the icon."));
   gtk_table_attach (GTK_TABLE (table), button, 0, 1, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
   gtk_widget_show (button);
 
@@ -346,9 +307,6 @@ thunar_preferences_dialog_init (ThunarPreferencesDialog *dialog)
       gtk_combo_box_append_text (GTK_COMBO_BOX (combo), date);
       g_free (date);
     }
-#if !GTK_CHECK_VERSION(2,9,0)
-  g_signal_connect (G_OBJECT (combo), "changed", G_CALLBACK (g_object_notify), "active");
-#endif
   exo_mutual_binding_new (G_OBJECT (dialog->preferences), "misc-date-style", G_OBJECT (combo), "active");
   gtk_table_attach (GTK_TABLE (table), combo, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
   thunar_gtk_label_set_a11y_relation (GTK_LABEL (label), combo);
@@ -393,9 +351,6 @@ thunar_preferences_dialog_init (ThunarPreferencesDialog *dialog)
   gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Large"));
   gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Larger"));
   gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Very Large"));
-#if !GTK_CHECK_VERSION(2,9,0)
-  g_signal_connect (G_OBJECT (combo), "changed", G_CALLBACK (g_object_notify), "active");
-#endif
   exo_mutual_binding_new_full (G_OBJECT (dialog->preferences), "shortcuts-icon-size", G_OBJECT (combo), "active",
                                transform_icon_size_to_index, transform_index_to_icon_size, NULL, NULL);
   gtk_table_attach (GTK_TABLE (table), combo, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
@@ -405,8 +360,8 @@ thunar_preferences_dialog_init (ThunarPreferencesDialog *dialog)
 
   button = gtk_check_button_new_with_mnemonic (_("Show Icon _Emblems"));
   exo_mutual_binding_new (G_OBJECT (dialog->preferences), "shortcuts-icon-emblems", G_OBJECT (button), "active");
-  thunar_gtk_widget_set_tooltip (button, _("Select this option to display icon emblems in the shortcuts pane for all folders "
-                                           "for which emblems have been defined in the folders properties dialog."));
+  gtk_widget_set_tooltip_text (button, _("Select this option to display icon emblems in the shortcuts pane for all folders "
+                                         "for which emblems have been defined in the folders properties dialog."));
   gtk_table_attach (GTK_TABLE (table), button, 0, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
   gtk_widget_show (button);
 
@@ -438,9 +393,6 @@ thunar_preferences_dialog_init (ThunarPreferencesDialog *dialog)
   gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Large"));
   gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Larger"));
   gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Very Large"));
-#if !GTK_CHECK_VERSION(2,9,0)
-  g_signal_connect (G_OBJECT (combo), "changed", G_CALLBACK (g_object_notify), "active");
-#endif
   exo_mutual_binding_new_full (G_OBJECT (dialog->preferences), "tree-icon-size", G_OBJECT (combo), "active",
                                transform_icon_size_to_index, transform_index_to_icon_size, NULL, NULL);
   gtk_table_attach (GTK_TABLE (table), combo, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
@@ -450,8 +402,8 @@ thunar_preferences_dialog_init (ThunarPreferencesDialog *dialog)
 
   button = gtk_check_button_new_with_mnemonic (_("Show Icon E_mblems"));
   exo_mutual_binding_new (G_OBJECT (dialog->preferences), "tree-icon-emblems", G_OBJECT (button), "active");
-  thunar_gtk_widget_set_tooltip (button, _("Select this option to display icon emblems in the tree pane for all folders "
-                                           "for which emblems have been defined in the folders properties dialog."));
+  gtk_widget_set_tooltip_text (button, _("Select this option to display icon emblems in the tree pane for all folders "
+                                         "for which emblems have been defined in the folders properties dialog."));
   gtk_table_attach (GTK_TABLE (table), button, 0, 2, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
   gtk_widget_show (button);
 
@@ -513,11 +465,11 @@ thunar_preferences_dialog_init (ThunarPreferencesDialog *dialog)
 
   range = gtk_hscale_new_with_range (0.0, 2000.0, 100.0);
   gtk_scale_set_draw_value (GTK_SCALE (range), FALSE);
-  thunar_gtk_widget_set_tooltip (range, _("When single-click activation is enabled, pausing the mouse pointer over an item "
-                                          "will automatically select that item after the chosen delay. You can disable this "
-                                          "behavior by moving the slider to the left-most position. This behavior may be "
-                                          "useful when single clicks activate items, and you want only to select the item "
-                                          "without activating it."));
+  gtk_widget_set_tooltip_text (range, _("When single-click activation is enabled, pausing the mouse pointer over an item "
+                                        "will automatically select that item after the chosen delay. You can disable this "
+                                        "behavior by moving the slider to the left-most position. This behavior may be "
+                                        "useful when single clicks activate items, and you want only to select the item "
+                                        "without activating it."));
   gtk_box_pack_start (GTK_BOX (ibox), range, FALSE, FALSE, 0);
   thunar_gtk_label_set_a11y_relation (GTK_LABEL (label), range);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), range);
@@ -592,9 +544,6 @@ thunar_preferences_dialog_init (ThunarPreferencesDialog *dialog)
   gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Ask everytime"));
   gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Apply to Folder Only"));
   gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _("Apply to Folder and Contents"));
-#if !GTK_CHECK_VERSION(2,9,0)
-  g_signal_connect (G_OBJECT (combo), "changed", G_CALLBACK (g_object_notify), "active");
-#endif
   exo_mutual_binding_new (G_OBJECT (dialog->preferences), "misc-recursive-permissions", G_OBJECT (combo), "active");
   gtk_table_attach (GTK_TABLE (table), combo, 0, 1, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
   thunar_gtk_label_set_a11y_relation (GTK_LABEL (label), combo);
@@ -616,52 +565,26 @@ thunar_preferences_dialog_init (ThunarPreferencesDialog *dialog)
   gtk_container_add (GTK_CONTAINER (frame), table);
   gtk_widget_show (table);
 
-  /* determine the active volume manager */
-  volume_manager = thunar_vfs_volume_manager_get_default ();
-
   /* check if "thunar-volman" is found */
   path = g_find_program_in_path ("thunar-volman");
 
-  /* check if we lack volume management support and should warn the user */
-  if (g_signal_lookup ("device-added", G_OBJECT_TYPE (volume_manager)) == 0 || path == NULL)
-    {
-      /* add a warning telling the user that volume management is not available */
-      image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_DND);
-      gtk_misc_set_alignment (GTK_MISC (image), 0.5f, 0.5f);
-      gtk_table_attach (GTK_TABLE (table), image, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
-      gtk_widget_show (image);
+  /* add check button to enable/disable auto mounting */
+  button = gtk_check_button_new_with_mnemonic (_("Enable _Volume Management"));
+  exo_mutual_binding_new (G_OBJECT (dialog->preferences), "misc-volume-management", G_OBJECT (button), "active");
+  gtk_table_attach (GTK_TABLE (table), button, 0, 1, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+  gtk_widget_show (button);
 
-      label = gtk_label_new ((path == NULL)
-          ? _("Install the \"thunar-volman\" package to use\nthe volume management support in Thunar.")
-          : _("Build thunar-vfs with HAL support to use\nthe volume management support in Thunar."));
-      gtk_misc_set_alignment (GTK_MISC (label), 0.0f, 0.5f);
-      gtk_table_attach (GTK_TABLE (table), label, 1, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
-      gtk_widget_show (label);
-
-      /* make sure to disable "misc-volume-management" then */
-      g_object_set (G_OBJECT (dialog->preferences), "misc-volume-management", FALSE, NULL);
-    }
-  else
-    {
-      /* add check button to enable/disable auto mounting */
-      button = gtk_check_button_new_with_mnemonic (_("Enable _Volume Management"));
-      exo_mutual_binding_new (G_OBJECT (dialog->preferences), "misc-volume-management", G_OBJECT (button), "active");
-      gtk_table_attach (GTK_TABLE (table), button, 0, 1, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
-      gtk_widget_show (button);
-
-      label = sexy_url_label_new ();
-      gtk_misc_set_alignment (GTK_MISC (label), 0.0f, 0.5f);
-      exo_binding_new (G_OBJECT (button), "active", G_OBJECT (label), "sensitive");
-      g_signal_connect_swapped (G_OBJECT (label), "url-activated", G_CALLBACK (thunar_preferences_dialog_configure), dialog);
-      /* TRANSLATORS: Make sure you place the <a>...</a>-link on the first line, otherwise the user will be unable to click on it */
-      sexy_url_label_set_markup (SEXY_URL_LABEL (label), _("<a href=\"volman-config:\">Configure</a> the management of removable drives\n"
-                                                           "and media (i.e. how cameras should be handled)."));
-      gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
-      gtk_widget_show (label);
-    }
+  label = sexy_url_label_new ();
+  gtk_misc_set_alignment (GTK_MISC (label), 0.0f, 0.5f);
+  exo_binding_new (G_OBJECT (button), "active", G_OBJECT (label), "sensitive");
+  g_signal_connect_swapped (G_OBJECT (label), "url-activated", G_CALLBACK (thunar_preferences_dialog_configure), dialog);
+  /* TRANSLATORS: Make sure you place the <a>...</a>-link on the first line, otherwise the user will be unable to click on it */
+  sexy_url_label_set_markup (SEXY_URL_LABEL (label), _("<a href=\"volman-config:\">Configure</a> the management of removable drives\n"
+                                                       "and media (i.e. how cameras should be handled)."));
+  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+  gtk_widget_show (label);
 
   /* cleanup */
-  g_object_unref (G_OBJECT (volume_manager));
   g_free (path);
 }
 
@@ -687,7 +610,8 @@ thunar_preferences_dialog_response (GtkDialog *dialog,
   if (G_UNLIKELY (response == GTK_RESPONSE_HELP))
     {
       /* open the preferences section of the user manual */
-      thunar_dialogs_show_help (GTK_WIDGET (dialog), "preferences", NULL);
+      xfce_dialog_show_help (GTK_WINDOW (dialog), "thunar",
+                             "preferences", NULL);
     }
   else
     {
@@ -733,11 +657,42 @@ thunar_preferences_dialog_configure (ThunarPreferencesDialog *dialog)
 GtkWidget*
 thunar_preferences_dialog_new (GtkWindow *parent)
 {
-  GtkWidget *dialog;
+  GtkWidget    *dialog;
+  gint          root_x, root_y;
+  gint          window_width, window_height;
+  gint          dialog_width, dialog_height;
+  gint          monitor;
+  GdkRectangle  geometry;
+  GdkScreen    *screen;
 
   dialog = g_object_new (THUNAR_TYPE_PREFERENCES_DIALOG, NULL);
+
+  /* fake gtk_window_set_transient_for() for centering the window on
+   * the parent window. See bug #3586. */
   if (G_LIKELY (parent != NULL))
-    gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
+    {
+      screen = gtk_window_get_screen (parent);
+      gtk_window_set_screen (GTK_WINDOW (dialog), screen);
+      gtk_widget_realize (dialog);
+
+      /* get the size and position on the windows */
+      gtk_window_get_position (parent, &root_x, &root_y);
+      gtk_window_get_size (parent, &window_width, &window_height);
+      gtk_window_get_size (GTK_WINDOW (dialog), &dialog_width, &dialog_height);
+
+      /* get the monitor geometry of the monitor with the parent window */
+      monitor = gdk_screen_get_monitor_at_point (screen, root_x, root_y);
+      gdk_screen_get_monitor_geometry (screen, monitor, &geometry);
+
+      /* center the dialog on the window and clamp on the monitor */
+      root_x += (window_width - dialog_width) / 2;
+      root_x = CLAMP (root_x, geometry.x, geometry.x + geometry.width - dialog_width);
+      root_y += (window_height - dialog_height) / 2;
+      root_y = CLAMP (root_y, geometry.y, geometry.y + geometry.height - dialog_height);
+
+      /* move the dialog */
+      gtk_window_move (GTK_WINDOW (dialog), root_x, root_y);
+    }
 
   return dialog;
 }
