@@ -1,6 +1,7 @@
 /* $Id$ */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
+ * Copyright (c) 2009 Jannis Pohlmann <jannis@xfce.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -24,7 +25,6 @@
 
 #include <thunarx/thunarx-file-info.h>
 #include <thunarx/thunarx-private.h>
-#include <thunarx/thunarx-alias.h>
 
 
 
@@ -45,30 +45,21 @@ static guint file_info_signals[LAST_SIGNAL];
 GType
 thunarx_file_info_get_type (void)
 {
-  static GType type = G_TYPE_INVALID;
+  static volatile gsize type__volatile = 0;
+  GType                 type;
 
-  if (G_UNLIKELY (type == G_TYPE_INVALID))
+  if (g_once_init_enter (&type__volatile))
     {
-      static const GTypeInfo info =
-      {
-        sizeof (ThunarxFileInfoIface),
-        NULL,
-        NULL,
-        NULL,
-        NULL,
-        NULL,
-        0,
-        0,
-        NULL,
-        NULL,
-      };
+      type = g_type_register_static_simple (G_TYPE_INTERFACE,
+                                            I_("ThunarxFileInfo"),
+                                            sizeof (ThunarxFileInfoIface),
+                                            NULL,
+                                            0,
+                                            NULL,
+                                            0);
 
-      /* register the interface type */
-      type = g_type_register_static (G_TYPE_INTERFACE, I_("ThunarxFileInfo"), &info, 0);
-
-      /* implementations must inherit GObject */
       g_type_interface_add_prerequisite (type, G_TYPE_OBJECT);
-
+      
       /**
        * ThunarxFileInfo::changed:
        * @file_info : a #ThunarxFileInfo.
@@ -108,9 +99,11 @@ thunarx_file_info_get_type (void)
                       NULL, NULL,
                       g_cclosure_marshal_VOID__VOID,
                       G_TYPE_NONE, 0);
+
+      g_once_init_leave (&type__volatile, type);
     }
 
-  return type;
+  return type__volatile;
 }
 
 
@@ -292,30 +285,67 @@ thunarx_file_info_is_directory (ThunarxFileInfo *file_info)
 
 
 /**
- * thunarx_file_info_get_vfs_info:
+ * thunarx_file_info_get_file_info:
  * @file_info : a #ThunarxFileInfo.
  *
- * Returns the #ThunarVfsInfo associated with @file_info,
+ * Returns the #GFileInfo associated with @file_info,
  * which includes additional information about the @file_info
- * as queried from the VFS library earlier. The caller is
- * responsible to free the returned #ThunarVfsInfo object
- * using thunar_vfs_info_unref() when no longer needed.
+ * as queried from GIO earlier. The caller is responsible to free the 
+ * returned #GFileInfo object using g_object_unref() when 
+ * no longer needed.
  *
- * Note that the <application>thunarx</application> library itself
- * is not linked to the <application>thunar-vfs</application> library,
- * and so, if you need to use this method, you'll need to include
- * <code>&lt;thunar-vfs/thunar-vfs.h&gt;</code> in your code and
- * add <code>`pkg-config --cflags thunar-vfs-1`</code> to your
- * <envar>CFLAGS</envar>.
- *
- * Return value: the #ThunarVfsInfo object associated with @file_info,
- *               which MUST be freed using thunar_vfs_info_unref().
+ * Return value: the #GFileInfo object associated with @file_info,
+ *               which MUST be freed using g_object_unref().
  **/
-ThunarVfsInfo*
-thunarx_file_info_get_vfs_info (ThunarxFileInfo *file_info)
+GFileInfo*
+thunarx_file_info_get_file_info (ThunarxFileInfo *file_info)
 {
   g_return_val_if_fail (THUNARX_IS_FILE_INFO (file_info), NULL);
-  return (*THUNARX_FILE_INFO_GET_IFACE (file_info)->get_vfs_info) (file_info);
+  return (*THUNARX_FILE_INFO_GET_IFACE (file_info)->get_file_info) (file_info);
+}
+
+
+
+/**
+ * thunarx_file_info_get_filesystem_info:
+ * @file_info : a #ThunarxFileInfo.
+ *
+ * Returns the #GFileInfo which includes additional information about
+ * the filesystem @file_info resides on. The caller is responsible to 
+ * free the returned #GFileInfo object using g_object_unref() when 
+ * no longer needed.
+ *
+ * Return value: the #GFileInfo containing information about the
+ *               filesystem of @file_info or %NULL if no filesystem 
+ *               information is available. It MUST be released using
+ *               g_object_unref().
+ **/
+GFileInfo*
+thunarx_file_info_get_filesystem_info (ThunarxFileInfo *file_info)
+{
+  g_return_val_if_fail (THUNARX_IS_FILE_INFO (file_info), NULL);
+  return (*THUNARX_FILE_INFO_GET_IFACE (file_info)->get_filesystem_info) (file_info);
+}
+
+
+
+/**
+ * thunarx_file_info_get_location:
+ * @file_info : a #ThunarxFileInfo.
+ *
+ * Returns the #GFile @file_info points to. The #GFile is a more
+ * powerful tool than just the URI or the path. The caller
+ * is responsible to release the returned #GFile using g_object_unref()
+ * when no longer needed.
+ *
+ * Return value: the #GFile to which @file_info points. It MUST be
+ *               released using g_object_unref().
+ **/
+GFile*
+thunarx_file_info_get_location (ThunarxFileInfo *file_info)
+{
+  g_return_val_if_fail (THUNARX_IS_FILE_INFO (file_info), NULL);
+  return (*THUNARX_FILE_INFO_GET_IFACE (file_info)->get_location) (file_info);
 }
 
 
@@ -391,8 +421,11 @@ thunarx_file_info_list_copy (GList *file_infos)
   GList *list = NULL;
   GList *lp;
 
-  for (lp = g_list_last (file_infos); lp != NULL; lp = lp->prev)
-    list = g_list_prepend (list, g_object_ref (G_OBJECT (lp->data)));
+  if (file_infos != NULL)
+    {
+      for (lp = g_list_last (file_infos); lp != NULL; lp = lp->prev)
+        list = g_list_prepend (list, g_object_ref (G_OBJECT (lp->data)));
+    }
 
   return list;
 }
@@ -410,11 +443,9 @@ thunarx_file_info_list_copy (GList *file_infos)
 void
 thunarx_file_info_list_free (GList *file_infos)
 {
-  g_list_foreach (file_infos, (GFunc) g_object_unref, NULL);
-  g_list_free (file_infos);
-}
-
-
-
-#define __THUNARX_FILE_INFO_C__
-#include <thunarx/thunarx-aliasdef.c>
+  if (file_infos != NULL)
+    {
+      g_list_foreach (file_infos, (GFunc) g_object_unref, NULL);
+      g_list_free (file_infos);
+    }
+}   
